﻿using System;
using System.Drawing;
using System.IO;
using CoreLib.Graphics;
using System.Drawing.Imaging;
using CoreLib;

namespace MapUtility
{
	public static class Extraction
	{
		public static void DefaultExtract()
		{
			Global.Log.WriteLine("Extracting from command -extract");
			if (!Directory.Exists(Shared.ExportedMapsDirectory))
			{
				Directory.CreateDirectory(Shared.ExportedMapsDirectory);
				Global.Log.WriteLine("Successfully created exported maps directory");
			}
			int num = Settings.HasFourLightmaps ? 8 : 4;
			using (StreamWriter sw = new StreamWriter(Shared.ExportedMapsDirectory + "\\" + Global.ImageListingPath))
			{
				for (int x = 0; x < num; x++)
				{
					for (int y = 0; y < num; y++)
					{
						int trueOffsetX = x * Settings.SmallestLmWidth;
						int trueOffsetY = y * Settings.SmallestLmHeight;
						sw.WriteLine("map_{0}_{1}.png;{0};{1}", trueOffsetX, trueOffsetY);
					}
				}
			}
			Global.Log.WriteLine("Successfully created default images.txt");
			ExtractImages(0, 0, Settings.SmallestLmWidth, Settings.SmallestLmHeight, num, num);
		}

		public static void ExtractImages(int offsetX, int offsetY, int imageSizeX, int imageSizeY, int numImagesX, int numImagesY)
		{
			Global.AdjacentTable.ToString(); //Force load

			Global.Log.WriteLine("Extracting images with following parameters: OffsetX={0} OffsetY={1} ImageSizeX={2} ImageSizeY={3} NumImagesX={4} NumImagesY={5}", offsetX, offsetY, imageSizeX, imageSizeY, numImagesX, numImagesY);
			Global.Log.WriteLine("Loading lightmap at {0}", Settings.GamePath + @"\map");
			Console.Write("Loading lightmap...");
			Lightmap map = new Lightmap(LightmapOption.LightMap1, Settings.GamePath + @"\map");
			Console.WriteLine("Done!");
			Lightmap.Scales = Lightmap.Colorscales.Plain;
			Global.Log.WriteLine("Lightmap1 successfully loaded.");
			map.LoadTileData();
			Global.Log.WriteLine("Lightmap1 tile data successfully loaded");

			TaskQueue extractJobs = new TaskQueue();

			for (int imagesX = 0; imagesX < numImagesX; imagesX++)
			{
				for (int imagesY = 0; imagesY < numImagesY; imagesY++)
				{
					int trueOffsetX = offsetX + imagesX * imageSizeX;
					int trueOffsetY = offsetY + imagesY * imageSizeY;
					Rectangle bounds = new Rectangle(trueOffsetX, trueOffsetY, imageSizeX, imageSizeY);
					if (bounds.X >= Settings.Lm1Width || bounds.Y >= Settings.Lm1Height)
						continue;
					if (bounds.Right > Settings.Lm1Width) bounds.Width = Settings.Lm1Width - bounds.X;
					if (bounds.Bottom > Settings.Lm1Height) bounds.Height = Settings.Lm1Height - bounds.Y;

					extractJobs.Add(new ExtractFileTask(bounds, map));

				}
				//GC.Collect();
				//GC.WaitForPendingFinalizers();
				//GC.Collect();
			}

			extractJobs.StartWork();
			extractJobs.FinishAll();

			Console.WriteLine();
			Console.WriteLine();
			Console.WriteLine("Done extracting images.");
			Global.Log.WriteLine("Finished extracting");
		}

		private static void ExtractSection(Lightmap lm, Rectangle bounds)
		{
			Console.WriteLine("Doing the following image: Offset=({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			Global.Log.WriteLine("Doing the following image: Offset=({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);

			ExtractGrayscale(lm, bounds);
			ExtractOwnershipImage(lm, bounds);
			ExtractTileImage(lm, bounds);
			ExtractBorders(lm, bounds);

			Console.WriteLine("Done doing the following image: Offset=({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			Global.Log.WriteLine("Done doing the following image: Offset=({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
		}

		private static void ExtractGrayscale(Lightmap map, Rectangle bounds)
		{
			Console.WriteLine("Extracting grayscale of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			Global.Log.WriteLine("Extracting grayscale of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			ImageData<RgbData> temp = map.GetTrueColoredGrayscaleLayer(bounds.X, bounds.Y, bounds.Width, bounds.Height);

			Bitmap bmp = temp.ConvertToGrayscale().InternalBitmap;

			bmp.Save(string.Format(@"{2}\map_{0}_{1}.colorscales.png",
				bounds.X, bounds.Y, Shared.ExportedMapsDirectory), System.Drawing.Imaging.ImageFormat.Png);
			bmp.Dispose();
			//Console.WriteLine("Done!");
		}
		private static void ExtractBorders(Lightmap map, Rectangle bounds)
		{
			Console.WriteLine("Extracting border image of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			Global.Log.WriteLine("Extracting border image of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			ImageData<ArgbData> a = map.GetBorderLayer(bounds.X, bounds.Y, bounds.Width, bounds.Height);
			if (a != null)
			{
				a.InternalBitmap.Data.Save(string.Format(@"{2}\map_{0}_{1}.borders.png",
					bounds.X, bounds.Y, Shared.ExportedMapsDirectory), System.Drawing.Imaging.ImageFormat.Png);
				a.InternalBitmap.Data.Dispose();
			}
			//Console.WriteLine("Done!");
		}
		private static void ExtractTileImage(Lightmap map, Rectangle bounds)
		{
			Console.WriteLine("Extracting tile image of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			Global.Log.WriteLine("Extracting tile image of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			ImageData<GrayscaleData> tilesLayerTemp = map.CreateTilesImage(bounds.X, bounds.Y, bounds.Width, bounds.Height);
			if (!tilesLayerTemp.AllIsSame())
			{
				ImageData<ArgbData> tilesLayer = new ImageData<ArgbData>(tilesLayerTemp.Width, tilesLayerTemp.Height);
				ColorPalette palette = tilesLayerTemp.InternalBitmap.Data.Palette;
				for (int i = 0; i < tilesLayer.Size; i++)
				{
					tilesLayer[i] = ArgbData.ToData(palette.Entries[tilesLayerTemp[i]]);
				}

				tilesLayer.InternalBitmap.Data.Save(string.Format(@"{2}\map_{0}_{1}.tiles.png",
					bounds.X, bounds.Y, Shared.ExportedMapsDirectory), System.Drawing.Imaging.ImageFormat.Png);
			}
			//Console.WriteLine("Done!");
		}
		private static void ExtractOwnershipImage(Lightmap map, Rectangle bounds)
		{
			Console.WriteLine("Extracting ownership image of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			Global.Log.WriteLine("Extracting ownership image of ({0},{1}), Size=({2},{3})",
				bounds.X, bounds.Y, bounds.Width, bounds.Height);
			Bitmap bmp = map.GetColoring<RgbData>
				(bounds.X, bounds.Y, bounds.Width, bounds.Height, Lightmap.ColorByProvinceId, false).InternalBitmap;

			bmp.Save(string.Format(@"{2}\map_{0}_{1}.ProvID.png",
				bounds.X, bounds.Y, Shared.ExportedMapsDirectory), System.Drawing.Imaging.ImageFormat.Png);
			bmp.Dispose();
			//Console.WriteLine("Done!");
		}

		class ExtractFileTask : Task
		{
			Lightmap lm;
			Rectangle bounds;

			public ExtractFileTask(Rectangle bounds, Lightmap lm)
				: base("aaa-iii")
			{
				this.lm = lm;
				this.bounds = bounds;
			}
			public override string Description
			{
				get { throw new NotImplementedException(); }
			}
			public override void TaskOverride()
			{
				ExtractSection(lm, bounds);
			}
		}
	}
}