﻿//#define TESTING

using System;
using System.IO;
using CoreLib;

namespace MapUtility
{
	class Program
	{
		static void Main(string[] args)
		{
			StreamWriter log = new StreamWriter("log.txt", false);

//#if !TESTING
			try
			{
//#endif
				log.WriteLine(Shared.ProductName + " Log");
				log.WriteLine("Recording started at " + DateTime.UtcNow.ToString());
				log.WriteLine();

				Global.SetLog(log);
				Global.Log.WriteLine("Successfully reached start");
				StartupWork(args);

//#if !TESTING
				AppDomain.CurrentDomain.UnhandledException += (sender, e) =>
				{
					OnException((Exception)e.ExceptionObject);
#if TESTING
					throw (Exception)e.ExceptionObject;
#endif
				};
			}
			catch (Exception ex)
			{
				OnException(ex);
#if TESTING
				throw;
#endif
			}
			finally
			{
				Global.Log.Close();
			}
//#endif
		}

		private static void OnException(Exception ex)
		{
			TextWriter log = Global.Log;
			Console.WriteLine("A fatal error has occurred. Send in log.txt. The application will now exit.");
			log.WriteLine();
			log.WriteLine();
			log.WriteLine("#################################");
			log.WriteLine("An exception has occurred.");
			log.WriteLine("#################################");
			#region Exception Type
			try
			{
				log.WriteLine("Exception: {0}", ex.GetType().FullName);
			}
			catch
			{
				log.WriteLine("Could not write exception type.");
			}
			#endregion
			#region DateTime
			try
			{
				log.WriteLine(DateTime.UtcNow);
			}
			catch
			{
				log.WriteLine("Failed to write date");
			}
			#endregion
			#region Message
			try
			{
				log.WriteLine("Message: " + ex.Message);
			}
			catch
			{
				log.WriteLine("Could not write Message!");
			}
			#endregion
			#region Source
			try
			{
				log.WriteLine("Source: " + ex.Source);
			}
			catch
			{
				log.WriteLine("Could not write Source!");
			}
			#endregion
			#region TargetSite
			try
			{
				log.WriteLine("Target Site: " + ex.TargetSite);
			}
			catch
			{
				log.WriteLine("Could not write Target Site!");
			}
			#endregion
			#region StackTrace
			try
			{
				log.WriteLine("Stack Trace: " + ex.StackTrace);
			}
			catch
			{
				log.WriteLine("Could not write Stack Trace!");
			}
			#endregion
			#region Data
			try
			{
				log.WriteLine("Misc data");
				foreach (object o in ex.Data)
					log.WriteLine(o);
			}
			catch
			{
				log.WriteLine("Could not write misc data");
			}
			#endregion
		}

		private static void StartupWork(string[] args)
		{
			Global.RootPath = Settings.GamePath + @"\map";

			Console.WriteLine(Shared.ProductName + " v" + AssemblyEx.GetVersion());
			Console.WriteLine();
			Console.WriteLine();

			if (args.Length != 0)
			{
				Global.Log.WriteLine("ARGUMENTS: ");
				for (int i = 0; i < args.Length; i++)
					Global.Log.WriteLine("Argument {0}: {1}", i.ToString(), args[i]);

				string checkPath = Settings.GamePath + @"\map\lightmap1.tbl";
				if (!File.Exists(checkPath))
				{
					Console.WriteLine("Lightmap1.tbl was not found. Check that GamePath in settings.txt is correct.");
					Console.WriteLine("Press any key to continue...");
					Global.Log.WriteLine("MapViewer is exiting because lightmap1.tbl was not found.");
					Global.Log.WriteLine("Searched path was {0}", checkPath);

					Console.ReadKey();
					return;
				}

				foreach (string arg in args)
				{
					#region Help
					const string HelpInfo =
					@"
COMMAND-LINE ARGUMENTS

-extract([offsetX],[offsetY],[imageSizeX],[imageSizeY],[numImagesX],[numImagesY])
Extracts parts of the map in .png files.
Blocks are groups of 32x32 pixels. There are {0}x{1} blocks in lightmap1.
offsetX, offsetY: The top-left image's offset from the top-left of the map in blocks.
imageSizeX, imageSizeY: The size of each image you want to extract.
numImagesX, numImagesY: How many images to extract in the x and y directions.
Warning: Extracting too much could lead to running out of memory and then crashing.
Note that there cannot be any spaces.

-extract
Same as -extract(0,0,72,72,13,5), but also creates a default images.txt.

-help: Shows this help

";
					if (arg.Length >= 5 && arg.Substring(0, 5).ToLower() == "-help")
					{
						Console.WriteLine(Shared.ProductName + " v" + AssemblyEx.GetVersion());
						Console.WriteLine();
						Console.Write(HelpInfo, Settings.Lm1Width, Settings.Lm1Height);

					}
					#endregion
					#region Extract
					if (arg.Length == 8 && arg == "-extract")
					{
						Extraction.DefaultExtract();
					}
					if (arg.Length > 8 && arg.Substring(0, 8).ToLower() == "-extract")
					{
						Global.Log.WriteLine("Ordered to do specific extract");
						string line = arg; //Because of ForEach
						line = line.Substring(8);
						line.ChangeTrim('(', ')');
						string[] vals = line.Split(',');
						int offsetX = int.Parse(vals[0]);
						int offsetY = int.Parse(vals[1]);
						int imageSizeX = int.Parse(vals[2]);
						int imageSizeY = int.Parse(vals[3]);
						int numImagesX = int.Parse(vals[4]);
						int numImagesY = int.Parse(vals[5]);
						Global.Log.WriteLine("Parse of arguments successful.");
						Extraction.ExtractImages(offsetX, offsetY, imageSizeX, imageSizeY, numImagesX, numImagesY);
					}
					#endregion
				}
				Global.Log.WriteLine("All arguments have been processed, and now exiting.");
			}
			else
			{
				Global.Log.WriteLine("Since there were no arguments, doing default extract");
				Extraction.DefaultExtract();
			}
		}

	}
}