﻿namespace MapEditor
{
	using CoreLib.Io;

	/// <summary>
	/// A class to store information about the location in a radish file of a block.
	/// </summary>
	public sealed class BlockHolder
	{
		#region Data
		private InnerBlockHolder data;
		/// <summary>
		/// Gets or sets the 0-based x-offset (in blocks) of the block to the top-left of the map.
		/// </summary>
		public ushort XOffset
		{
			get
			{
				return data.XOffset;
			}
			set
			{
				data.XOffset = value;
			}
		}

		/// <summary>
		/// Gets or sets the 0-based y-offset (in blocks) of the block to the top-left of the map.
		/// </summary>
		public ushort YOffset
		{
			get
			{
				return data.YOffset;
			}
			set
			{
				data.YOffset = value;
			}
		}

		public int Length
		{
			get
			{
				return data.Length;
			}
			set
			{
				data.Length = value;
			}
		}

		/// <summary>
		/// Gets or sets the path of the radish file where this block is contained.
		/// </summary>
		public string Path
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets the position of the start of the Block in the radish file, including the 4-byte Length field.
		/// </summary>
		public int Position
		{
			get { return data.Position; }
			set { data.Position = value; }
		}
		#endregion

		#region Creation
		/// <summary>
		/// .Ctor().
		/// </summary>
		/// <param name="xOffset">The x-offset.</param>
		/// <param name="yOffset">The y-offset.</param>
		/// <param name="path">The path.</param>
		/// <param name="position">The position.</param>
		public BlockHolder(ushort xOffset, ushort yOffset, string path, int position)
		{
			this.XOffset = xOffset;
			this.YOffset = yOffset;
			this.Path = path;
			this.Position = position;
		}
		private BlockHolder(string path)
		{
			this.Path = path;
		}
		
		/*/// <summary>
		/// Given a stream, creates a blockholder.
		/// </summary>
		/// <param name="stream">The stream to read from.</param>
		/// <param name="path">The file name of the stream.</param>
		/// <returns>The blockholder representing the block.</returns>
		public unsafe static BlockHolder MakeBlockHolder(BinaryStream stream, string path)
		{
			int pos = stream.Position + 4;

			BlockHolder ret = new BlockHolder(pos, path);

			fixed (byte* p = &stream.ReadBytes(8)[0])
			{
				ret.data = *(InnerBlockHolder*)p;
			}

			stream.Position = pos + ret.data.length;

			return ret;
		}*/

		public unsafe static BlockHolder MakeBlockHolder(IStreamReader stream, string path)
		{
			BlockHolder ret = new BlockHolder(path);

			fixed (byte* p = &stream.ReadBytes(12)[0])
			{
				ret.data = *(InnerBlockHolder*)p;
			}
			return ret;
		}
		#endregion

		/// <summary>
		/// Returns a string representation of the class.
		/// </summary>
		/// <returns>A string representing the class.</returns>
		public override string ToString()
		{
			return string.Format("({0}, {1}): {2}, {3}", this.XOffset, this.YOffset, this.Position, this.Path);
		}
		
		struct InnerBlockHolder
		{
			public ushort XOffset;
			public ushort YOffset;
			public int Position;
			public int Length;
		}
	}
}