﻿using System.Collections.Generic;
using System.Diagnostics;
using System.IO;

namespace MapEditor
{
	static class BorderProvinces
	{
		private static object Lock = new object();

		private const string Path = Shared.TempFolder + @"borders.txt";

		private static bool loaded = false;
		private static List<ushort> provs = new List<ushort>();
		[DebuggerHidden]
		public static bool Contains(ushort id)
		{
			return provs.Contains(id);
		}
		/// <summary>
		/// Binary searches. This assumes that the list has already been sorted.
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		public static bool SortedContains(ushort id)
		{
			return provs.BinarySearch(id) >= 0;
		}
		public static void Add(ushort id)
		{
			lock (Lock)
			{
				provs.Add(id);
			}
		}
		/// <summary>
		/// Adds the id if it is not yet listed.
		/// </summary>
		/// <param name="id"></param>
		public static void TryAdd(ushort id)
		{
			lock (Lock)
			{
				if (!provs.Contains(id))
					provs.Add(id);
			}
		}

		public static void Save()
		{
			lock (Lock)
			{
				if (File.Exists(Path))
					File.Delete(Path);
				provs.Sort();
				using (StreamWriter sw = new StreamWriter(File.Create(Path)))
				{
					foreach (ushort u in provs)
					{
						sw.WriteLine(u);
					}
				}
			}
		}
		public static void Load()
		{
			if (loaded)
				return;
			lock (Lock)
			{
				if (loaded)
					return;
				if (File.Exists(Path))
				{
					using (StreamReader sr = new StreamReader(Path))
					{
						while (true)
						{
							string s = sr.ReadLine();
							if (string.IsNullOrEmpty(s))
								return;
							provs.Add(ushort.Parse(s));
						}
					}
				}
				loaded = true;
			}
		}
		public static void Reset()
		{
			lock (Lock)
			{
				loaded = true;
				provs.Clear();
			}
		}
	}
}
