﻿using System;
using System.Diagnostics;
using CoreLib.Graphics;
using CoreLib.Io;

namespace MapEditor
{
	public sealed class ColoringData : CompressableBlockBase<byte>
	{
		#region Constructors
		public ColoringData(bool compressed) : base(compressed) { }
		//Compressed, gives value
		public ColoringData(byte value) : base(value) { }
		public ColoringData(byte[] values) : base(values) { }
		public ColoringData(byte[] values, bool checkCompression) : base(values, checkCompression) { }
		public ColoringData(GrayscaleData[] values)
			: this(Array.ConvertAll(values, color => color.Strength), false)
		{
		}
		public ColoringData(GrayscaleData[] values, bool checkCompression)
			: this(Array.ConvertAll(values, color => color.Strength), checkCompression)
		{
		}
		#endregion

		#region Streaming
		public static ColoringData Read(IStreamReader stream)
		{
			if (stream.ReadBool())
			{
				return new ColoringData(stream.ReadByte());
			}
			else
			{
				return new ColoringData(stream.ReadBytes(1024));
			}
		}
		[Obsolete]
		public static ColoringData Read(ByteReader stream)
		{
			if (stream.ReadBool())
			{
				return new ColoringData(stream.ReadByte());
			}
			else
			{
				return new ColoringData(stream.ReadBytes(1024));
			}
		}
		public int Write(IStreamWriter stream)
		{
			TryToCompress();
			stream.WriteBool(Compressed);
			if (Compressed)
			{
				stream.WriteByte(value);
			}
			else
			{
				stream.WriteBytes(data);
			}
			return (Compressed ? 2 : 1025);
		}
		#endregion

		public unsafe void LoadDataFrom(byte* data, bool checkCompression)
		{
			this.data = new byte[1024];
			fixed (byte* p = &this.data[0])
			{
				CoreLib.Interop.Win32.Kernel.CopyMemory((IntPtr)p, (IntPtr)data, 1024);
			}
			if (checkCompression)
				TryToCompress();
		}
	}
}