﻿#define TESTING
//#define TIMER

using System;
using System.Collections.Generic;
using CoreLib;
using CoreLib.Io;
using Directory = System.IO.Directory;
using System.Drawing;
using StreamWriter = System.IO.StreamWriter;

using MapUtility;

namespace MapEditor
{
#if TIMER
	class Watch
	{
		private DateTime start;
		private DateTime end;
		public void Start()
		{
			start = DateTime.Now;
			Console.WriteLine("Job started at {0}", start);
		}
		public void Stop()
		{
			end = DateTime.Now;
			Console.WriteLine("Job finished at {0}", end);
			Console.WriteLine("Total time elapsed was {0}", end - start);
		}
	}
#endif

	public static class Program
	{
		private static void EnsureDirectoriesExist()
		{
			Console.Write("Checking directories...");
			if (!Directory.Exists(Shared.OutputFolder))
				Directory.CreateDirectory(Shared.OutputFolder);
			if (!Directory.Exists(Shared.TempFolder))
				Directory.CreateDirectory(Shared.TempFolder);
			if (!Directory.Exists(Shared.RadishFolder))
				Directory.CreateDirectory(Shared.RadishFolder);
			Console.WriteLine("Done!");
		}
		private static List<LineData> LoadFiles()
		{
			Console.Write("Loading data from images.txt...");
			if (!File.Exists(Global.ImageListingPath))
			{
				ConsoleEx.ReportFatalError(string.Format
					(@"There must be a file in the same place as this application, called {0}, listing all of the image files.
(As per instructions in the MapMaker Data Specifications)",
					Global.ImageListingPath));
				Console.WriteLine("Press any key to exit...");
				Console.ReadKey();
				return null;
			}
			List<string> lines = File.ReadAndCleanLines(Global.ImageListingPath);
			List<LineData> files = new List<LineData>();
			foreach (string s in lines)
				files.Add(LineData.Interpret(s));
			Console.WriteLine("Done!");
			return files;
		}
		public static void BeginWork(string[] args)
		{
			//Global.RootPath = @"D:\Doomsday\Vanilla Lite\map";
			//Global.AdjacentTable.ToString();
			//NavalDist dist = new NavalDist(@"D:\Doomsday\Vanilla Lite\map\navaldist.tbl");
			//dist.ExportData(@"C:\Users\Jamie\Desktop\MapUtility\Information\Map Info\data.txt");

			Console.WriteLine("{0} v{1}", Shared.ProductName, AssemblyEx.GetVersion());
			Console.WriteLine();

			Console.WriteLine("Initializing...");
			EnsureDirectoriesExist();
			List<LineData> files = LoadFiles();
			if (files == null) return;
			Console.WriteLine("...Done!");

			while (true)
			{
				#region Display options
				Console.WriteLine();
				Console.WriteLine("What do you want to do now?");
				Console.WriteLine("0) Exit");
				Console.WriteLine("1) Make initial radish files");
				Console.WriteLine("2) Make other radish files");
				Console.WriteLine("3) Make adjacent.tbl");
				Console.WriteLine("4) Make index.tbl and tiles.bmp");
				Console.WriteLine("5) Make the lightmaps");
				Console.WriteLine("6) Make boundbox.tbl");
				Console.WriteLine();
				Console.WriteLine("Other options:");
				Console.WriteLine("7) Rebuild a certain radish file");
				Console.WriteLine("8) Rebuild a certain radish file level");
				Console.WriteLine();
				Console.WriteLine("Enter the number of the choice that you want");
				#endregion
				#region Finding selected number
				string s = Console.ReadLine();
				int val;
				if (!int.TryParse(s, out val))
				{
					Console.WriteLine("Invalid choice, please enter another.");
					continue;
				}
				#endregion
				try
				{
#if TIMER
					Watch w = new Watch();
#endif
					#region Options
					switch (val)
					{
						#region Exit
						case 0:
							Console.WriteLine("Goodbye!");
							Console.Write("Press any key to exit...");
							Console.ReadKey(true);
							return;
						#endregion
						#region Make initial radish files
						case 1:
#if TIMER
							w.Start();
#endif
							BorderProvinces.Reset();
							RadishMaker.MakeFiles(files);
							Console.WriteLine("Now checking that all blocks are properly available...");
							Utility.EnsureAllBlocksExist();
#if TIMER
							w.Stop();
#endif
							continue;
						#endregion
						#region Make other radish files
						case 2: //Make other radish files
#if TIMER
							w.Start();
#endif
							RadishHelper.MakeOtherRadishes();
#if TIMER
							w.Stop();
#endif
							break;
						#endregion
						#region Make adjacent.tbl
						case 3:
#if TIMER
							w.Start();
#endif
							AdjacentMaker.MakeAdjacent();
#if TIMER
							w.Stop();
#endif
							continue;
						#endregion
						#region Make tiles
						case 4:
#if TIMER
							w.Start();
#endif
							TileMaker.MakeTiles();
#if TIMER
							w.Stop();
#endif
							break;
						#endregion
						#region Make lightmaps
						case 5:
#if TIMER
							w.Start();
#endif
							LightmapMaker.MakeLightmaps();
#if TIMER
							w.Stop();
#endif
							break;
						#endregion
						#region Make bound boxes
						case 6:
#if TIMER
							w.Start();
#endif
							BoundBoxMaker.MakeBoundBoxes();
#if TIMER
							w.Stop();
#endif
							continue;
						#endregion
						#region Make a certain initial radish file
						case 7:
							for (int i = 0; i < files.Count; i++)
							{
								Console.WriteLine(i.ToString() + ")" + files[i].FilePath);
							}
							Console.WriteLine();
							int num;
							if (!int.TryParse(Console.ReadLine(), out num))
							{
								Console.WriteLine("Invalid input!");
								continue;
							}
#if TIMER
							w.Start();
#endif
							RadishMaker.MakeFile(files[num]);
#if TIMER
							w.Stop();
#endif
							continue;
						#endregion
						#region Make certain level radishes
						case 8:
							Console.WriteLine("Which level? Select 1 to " + (Settings.HasFourLightmaps ? "4" : "3"));
							int num1;
							if (!int.TryParse(Console.ReadLine(), out num1))
							{
								Console.WriteLine("Invalid input!");
								continue;
							}
#if TIMER
							w.Start();
#endif
							if (num1 == 1)
								RadishMaker.CombineRadishes();
							else if (num1 == 2)
								RadishHelper.MakeLightmap2Radish();
							else if (num1 == 3)
								RadishHelper.MakeLightmap3Radish();
							else if (num1 == 4 && Settings.HasFourLightmaps)
								RadishHelper.MakeLightmap4Radish();
							else
								Console.WriteLine("Invalid input!");
#if TIMER
							w.Stop();
#endif
							break;
						#endregion
						default:
							Console.WriteLine("Invalid choice, please enter another");
							continue;
					}
					#endregion
				}
				#region IO error catching
				catch (System.IO.IOException ioEx)
					//(System.ExecutionEngineException ioEx)
				{
					Console.WriteLine();
					Console.WriteLine();
					Console.WriteLine("An IO error has occured.");
					Console.WriteLine(ioEx.Message);
					Console.WriteLine("Press any key to continue...");
					Console.ReadKey();
				}
				#endregion
			}
		}
	}
	public static class Boot
	{
		public static void Main(string[] args)
		{
			using (StreamWriter log = new StreamWriter("log.txt", false))
			{
				log.WriteLine(Shared.ProductName + " Log");
				log.WriteLine("Recording started at " + DateTime.UtcNow.ToString());
				log.WriteLine();

				AppDomain.CurrentDomain.UnhandledException += (sender, e) =>
				{
					OnException((Exception)e.ExceptionObject);
#if TESTING
					throw (Exception)e.ExceptionObject;
#endif
				};
				Global.SetLog(log);
				try
				{
					Program.BeginWork(args);
				}
				catch (Exception e)
				{
					OnException(e);
#if TESTING
					throw;
#endif
				}
			}
		}

		static void OnException(Exception e)
		{
			Global.Log.WriteLine(DateTime.UtcNow);
			Console.WriteLine("A fatal error has occurred! Send log.txt in");
			try
			{
				Global.Log.WriteLine("Message: " + e.Message);
			}
			catch
			{
				Global.Log.WriteLine("Could not write Message!");
			}
			try
			{
				Global.Log.WriteLine("Source: " + e.Source);
			}
			catch
			{
				Global.Log.WriteLine("Could not write Source!");
			}
			try
			{
				Global.Log.WriteLine("Target Site: " + e.TargetSite);
			}
			catch
			{
				Global.Log.WriteLine("Could not write Target Site!");
			}
			try
			{
				Global.Log.WriteLine("Stack Trace: " + e.StackTrace);
			}
			catch
			{
				Global.Log.WriteLine("Could not write Stack Trace!");
			}
			try
			{
				Global.Log.WriteLine("Misc");
				foreach (object o in e.Data)
					Global.Log.WriteLine(o);
			}
			catch
			{
				Global.Log.WriteLine("Could not write misc data");
			}
			Global.Log.Flush();
		}
	}
}