﻿using System;
using System.Linq;
using CoreLib.Io;
using CoreLib;

using MapUtility;

namespace MapEditor
{
	public static class Shared
	{
		#region File Paths
		/// <summary>
		/// The output folder. This will contain the .tbl files and tiles.bmp.
		/// </summary>
		public const string OutputFolder = @"map\";
		/// <summary>
		/// The radish files folder. This will contain the zoom 1 radish files.
		/// </summary>
		public const string RadishFolder = @"radish\";
		/// <summary>
		/// The temp folder. This will contain the zoom 2, 3, and 4 radish files.
		/// </summary>
		public const string TempFolder = @"temp\";
		public const string AdjacentPath = OutputFolder + "adjacent.tbl";
		public const string Lightmap1Path = OutputFolder + "lightmap1.tbl";
		public const string Lightmap2Path = OutputFolder + "lightmap2.tbl";
		public const string Lightmap3Path = OutputFolder + "lightmap3.tbl";
		public const string Lightmap4Path = OutputFolder + "lightmap4.tbl";
		public const string BoundboxPath = OutputFolder + "boundbox.tbl";
		public const string IndexPath = OutputFolder + "index.tbl";
		public const string TilesPath = OutputFolder + "tiles.bmp";
		public const string Lightmap1RadishPath = TempFolder + "1.radish";
		public const string Lightmap2RadishPath = TempFolder + "2.radish";
		public const string Lightmap3RadishPath = TempFolder + "3.radish";
		public const string Lightmap4RadishPath = TempFolder + "4.radish";
		#endregion

		#region Image Names
		public const string BaseImageName = "Base";
		public const string ProvIdImageName = "ProvID";
		public const string GrayscaleImageName = "Colorscales";
		public const string TilesImageName = "Tiles";
		public const string BordersImageName = "Borders";
		#endregion

		#region Block Headers
		public static BlockHolder[,] LoadBlockHeaders(int blocksX, int blocksY, params string[] files)
		{
			BlockHolder[,] blocks = new BlockHolder[blocksX, blocksY]; //Holds the blocks
			Console.Write("Loading block headers from ");
			int maxLength = files.Max(s => s.Length) + 3;
			foreach (string path in files)
			{
				Console.Write(path + "...");
				for (int i = path.Length + 3; i < maxLength; i++)
					Console.Write(' ');
				using (BinaryStream stream =
					new BinaryStream(path.Substring(0, path.LastIndexOf('.')) + Shared.HeaderExtension, System.IO.FileMode.Open))
				{
					int streamLength = stream.Length;
					while (stream.Position < streamLength)
					{
						BlockHolder newBlock = BlockHolder.MakeBlockHolder(stream, path);
						blocks[newBlock.XOffset, newBlock.YOffset] = newBlock;
					}
				}
				ConsoleEx.Backspace(maxLength);
			}
			ConsoleEx.Backspace(6);
			Console.Write("...Done!");
			Console.WriteLine();
			return blocks;
		}
		private static BlockHolder[,] lightmap1RadishHeaders;
		private static BlockHolder[,] lightmap2RadishHeaders;
		private static BlockHolder[,] lightmap3RadishHeaders;
		private static BlockHolder[,] lightmap4RadishHeaders;
		public static BlockHolder[,] Lightmap1RadishHeaders
		{
			get
			{
				return LoadRadishHeaders(Settings.Lm1Width, Settings.Lm1Height, Shared.Lightmap1RadishPath, ref lightmap1RadishHeaders);
			}
			set { lightmap1RadishHeaders = value; }
		}
		public static BlockHolder[,] Lightmap2RadishHeaders
		{
			get
			{
				return LoadRadishHeaders(Settings.Lm2Width, Settings.Lm2Height, Shared.Lightmap2RadishPath, ref lightmap2RadishHeaders);
			}
		}
		public static BlockHolder[,] Lightmap3RadishHeaders
		{
			get
			{
				return LoadRadishHeaders(Settings.Lm3Width, Settings.Lm3Height, Shared.Lightmap3RadishPath, ref lightmap3RadishHeaders);
			}
		}
		public static BlockHolder[,] Lightmap4RadishHeaders
		{
			get
			{
				return LoadRadishHeaders(Settings.Lm4Width, Settings.Lm4Height, Shared.Lightmap4RadishPath, ref lightmap4RadishHeaders);
			}
		}
		private static BlockHolder[,] LoadRadishHeaders(int width, int height, string path, ref BlockHolder[,] headers)
		{
			if (headers == null)
			{
				headers = LoadBlockHeaders(width, height, path);
			}
			return headers;
		}
		#endregion

		#region Product Info
		public const string ProductName = "MapMaker";
		#endregion

		public const string HeaderExtension = ".headers";
	}
}