﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using CoreLib.Graphics;
using CoreLib.Io;
using MapUtility;
using CoreLib;

namespace MapEditor
{
	public static class Utility
	{
		/// <summary>
		/// Makes sure that all blocks are available.
		/// </summary>
		/// <param name="files"></param>
		/// <param name="maxX"></param>
		/// <param name="maxY"></param>
		public static void CheckRadishFileIds(string[] files, int maxX, int maxY)
		{
			//List<Point> all = new List<Point>(maxX * maxY);
			BitArray bits = new BitArray(maxX * maxY);
			foreach (string s in files)
			{
				using (ByteReader br = new ByteReader(s, true))
				{
					while (br.Position < br.Length)
					{
						int length = br.ReadInt32();
						int pos = br.Position;
						int xoffset = br.ReadUInt16();
						int yoffset = br.ReadUInt16();
						bits[yoffset * maxX + xoffset] = true;
						//all.Add(new Point(xoffset, yoffset));
						br.Position = length + pos;
					}
				}
			}
			List<Point> missing = new List<Point>();
			for (int x = 0; x < maxX; x++)
				for (int y = 0; y < maxY; y++)
				{
					if (!bits[y * maxX + x])
						missing.Add(new Point(x, y));
				}
			int loop = 0;
			foreach (Point p in missing)
			{
				Console.WriteLine(p.ToString());
				loop++;
				if (loop == 50)
				{
					loop = 0;
					Console.ReadLine();
				}
			}
			Console.WriteLine("Done");
			Console.ReadLine();
		}

		/*static Utility()
		{
			idToColor = new IdToColor();
			for (ushort id = 0; id < 4000; id++)
			{
				int result = ((((((((((((id & 0x200) >> 1) | ((id & 0x40) << 3)) | ((id & 8) << 7))
					| ((id & 1) << 11)) | ((id & 0x400) >> 6)) | ((id & 0x80) >> 2)) | ((id & 0x10) << 2)) |
					((id & 2) << 6)) | ((id & 0x800) >> 11)) | ((id & 0x100) >> 7)) | ((id & 0x20) >> 3)) | ((id & 4) << 1);
				result = (((result & 0xf00) << 12) | ((result & 240) << 8)) | ((result & 15) << 4);
				result = (result | -16777216);
				idToColor.provinceColors.Add(id, Color.FromArgb(result));
			}
		}*/

		public static void ExtractBlock(int x, int y)
		{
			ProtoBlock b = RadishHelper.LoadBlockData(x, y, Shared.Lightmap1RadishHeaders, null);
			ImageData<GrayscaleData> grayscale = new ImageData<GrayscaleData>(32, 32);
			for (int i = 0; i < 1024; i++)
				grayscale[i] = b.Grayscale[i];
			grayscale.InternalBitmap.Data.Save("temp\\grayscale.png", System.Drawing.Imaging.ImageFormat.Png);
			ImageData<RgbData> provid = new ImageData<RgbData>(32, 32);
			for (int i = 0; i < 1024; i++)
				provid[i] = Global.IdToColor[b.Ownership[i].OwnerId];
			provid.InternalBitmap.Data.Save("temp\\provid.png", System.Drawing.Imaging.ImageFormat.Png);
			ImageData<ArgbData> border = new ImageData<ArgbData>(32, 32);
			for (int i = 0; i < 1024; i++)
			{
				if (b.Ownership[i].IsBorder)
				{
					if (b.Ownership[i].IsTypeOne)
					{
						border[i] = RgbData.Red;
					}
					else
					{
						border[i] = RgbData.Blue;
					}
				}
			}
			border.InternalBitmap.Data.Save("temp\\border.png", System.Drawing.Imaging.ImageFormat.Png);
		}

		public static void ExtractBlockData(BlockHolder[,] holders, int x, int y)
		{
			ProtoBlock b = RadishHelper.LoadBlockData(x, y, holders, null);
			using (StreamWriter stream = new StreamWriter(string.Format("temp\\data{0}_{1}.txt", x, y)))
			{
				for (int i = 0; i < 1024; i++)
				{
					stream.Write("X: {0} Y: {1}\t", i / 32, i % 32);
					stream.WriteLine("Grayscale: {0} OwnerID: {1} IsBorder: {2} \tOtherId: {3}", b.Grayscale[i], b.Ownership[i].OwnerId,
						b.Ownership[i].IsBorder, b.Ownership[i].OtherOwner);
				}
			}
		}

		public static void MakeIdsCsv()
		{
			using (StreamWriter sw = new StreamWriter(new FileStream("aaa.csv", FileMode.Create)))
			{
				for (ushort id = 0; id < 4000; id++)
				{
					int result = ((((((((((((id & 0x200) >> 1) | ((id & 0x40) << 3)) |
						((id & 8) << 7)) | ((id & 1) << 11)) | ((id & 0x400) >> 6)) | ((id & 0x80) >> 2)) |
						((id & 0x10) << 2)) | ((id & 2) << 6)) | ((id & 0x800) >> 11)) | ((id & 0x100) >> 7)) |
						((id & 0x20) >> 3)) | ((id & 4) << 1);
					result = (((result & 0xf00) << 12) | ((result & 240) << 8)) | ((result & 15) << 4);
					result = (result | -16777216);
					Color c = Color.FromArgb(result);
					string r = c.R.ToString();
					string g = c.G.ToString();
					string b = c.B.ToString();
					sw.WriteLine(id.ToString() + ";" + r + ";" + g + ";" + b);
				}
			}
		}

		public static void EnsureAllBlocksExist()
		{
			Console.WriteLine("Beginning check...");
			Console.WriteLine("(The results will be dumped into check.txt)");

			string[] files = Directory.GetFiles(Shared.RadishFolder, "*" + Shared.HeaderExtension);

			bool[,] exists = new bool[Settings.Lm1Width, Settings.Lm1Height];
			BlockHolder[,] current = new BlockHolder[Settings.Lm1Width, Settings.Lm1Height];

			bool anyError = false;

			using (StreamWriter log = new StreamWriter("check.txt", false))
			{
				Console.Write("Scanning ");
				foreach (string path in files)
				{
					Console.Write(path);
					using (BinaryStream read = new BinaryStream(path, FileMode.Open))
					{
						while (read.Position != read.Length)
						{
							BlockHolder newBlock = BlockHolder.MakeBlockHolder(read, path);
							if (exists[newBlock.XOffset, newBlock.YOffset])
							{
								anyError = true;
								log.WriteLine("Duplicate block!");
								log.WriteLine("At ({0}, {1})", newBlock.XOffset.ToString(), newBlock.YOffset.ToString());
								log.WriteLine("First file: " + current[newBlock.XOffset, newBlock.YOffset].Path);
								log.WriteLine("Second file: " + newBlock.Path);
								log.WriteLine();
							}
							else
							{
								exists[newBlock.XOffset, newBlock.YOffset] = true;
								current[newBlock.XOffset, newBlock.YOffset] = newBlock;
							}
						}
					}
					ConsoleEx.Backspace(path.Length);
				}
				Console.WriteLine("\b...Done!");

				for (int x = 0; x < Settings.Lm1Width; x++)
				{
					for (int y = 0; y < Settings.Lm1Height; y++)
					{
						if (!exists[x, y])
						{
							anyError = true;
							log.WriteLine("Missing block at ({0}, {1})", x.ToString(), y.ToString());
						}
					}
				}
			}
			if (anyError)
				Console.WriteLine("There were errors with the radish files! Check check.txt for details.");
			else
			{
				Console.WriteLine("There were no errors, congratulations!");
				System.IO.File.Delete("check.txt");
			}
		}

		public static void CompareAdjancies(string file1, string file2)
		{
			AdjacentTable first = new AdjacentTable(file1);
			AdjacentTable second = new AdjacentTable(file2);

			int count = 0;
			for (ushort i = 0; i < first.NumProvinces; i++)
				count += first[i].NumConnections;
			Console.WriteLine("Count: " + count.ToString());
			Console.WriteLine();

			for (ushort i = 0; i < first.NumProvinces; i++)
			{
				AdjacentProvinceInfo top = first[i];
				AdjacentProvinceInfo bottom = second[i];

				bool areSame = true;
				if (top.NumConnections != bottom.NumConnections)
				{
					areSame = false;
				}
				else
				{
					for (int ia = 0; ia < top.NumConnections; ia++)
					{
						if (top[ia] != bottom[ia])
						{
							areSame = false;
							break;
						}
					}
				}

				if (!areSame)
				{
					Console.WriteLine("Province {0}", i);
					Console.WriteLine("First: ");
					foreach (ushort u in top)
					{
						if (!bottom.Contains(u))
						{
							Console.WriteLine(u);
						}
					}
					Console.WriteLine("Second: ");
					foreach (ushort u in bottom)
					{
						if (!top.Contains(u))
						{
							Console.WriteLine(u);
						}
					}
					Console.WriteLine();

					Console.ReadKey(true);
				}
			}
		}
	}
}