﻿//#define TESTING

using System;
using System.IO;
using System.Windows;
using MapUtility;

namespace ObjectEditor
{
	/// <summary>
	/// Interaction logic for App.xaml
	/// </summary>
	public partial class App : Application
	{
		protected override void OnStartup(StartupEventArgs e)
		{
			base.OnStartup(e);

			StreamWriter log = new StreamWriter("log.txt", false);

//#if !TESTING
			try
			{
//#endif
				log.WriteLine(Shared.ProductName + " Log");
				log.WriteLine("Recording started at " + DateTime.UtcNow.ToString());
				log.WriteLine();

				Global.SetLog(log);
				Global.Log.WriteLine("Successfully reached start");
				Global.RootPath = Settings.GamePath + @"\map";

//#if !TESTING
				this.DispatcherUnhandledException += (sender, eventArgs) =>
				{
					OnException(eventArgs.Exception);
#if !TESTING
						eventArgs.Handled = true;
#endif
				};
				AppDomain.CurrentDomain.UnhandledException += (sender, eventArgs) =>
				{
					OnException((Exception)eventArgs.ExceptionObject);
#if TESTING
					throw (Exception)eventArgs.ExceptionObject;
#endif
				};
			}
			catch (Exception ex)
			{
				OnException(ex);
#if TESTING
				throw;
#endif
			}
//#endif
		}
		private void OnException(Exception ex)
		{
			TextWriter log = Global.Log;
			MessageBox.Show("A fatal error has occurred. Send in log.txt. The application will now exit.");
			log.WriteLine();
			log.WriteLine();
			log.WriteLine("#################################");
			log.WriteLine("An exception has occurred.");
			log.WriteLine("#################################");
			#region Exception Type
			try
			{
				log.WriteLine("Exception: {0}", ex.GetType().FullName);
			}
			catch
			{
				log.WriteLine("Could not write exception type.");
			}
			#endregion
			#region DateTime
			try
			{
				log.WriteLine(DateTime.UtcNow);
			}
			catch
			{
				log.WriteLine("Failed to write date");
			}
			#endregion
			#region Message
			try
			{
				log.WriteLine("Message: " + ex.Message);
			}
			catch
			{
				log.WriteLine("Could not write Message!");
			}
			#endregion
			#region Source
			try
			{
				log.WriteLine("Source: " + ex.Source);
			}
			catch
			{
				log.WriteLine("Could not write Source!");
			}
			#endregion
			#region TargetSite
			try
			{
				log.WriteLine("Target Site: " + ex.TargetSite);
			}
			catch
			{
				log.WriteLine("Could not write Target Site!");
			}
			#endregion
			#region StackTrace
			try
			{
				log.WriteLine("Stack Trace: " + ex.StackTrace);
			}
			catch
			{
				log.WriteLine("Could not write Stack Trace!");
			}
			#endregion
			#region Data
			try
			{
				log.WriteLine("Misc data");
				foreach (object o in ex.Data)
					log.WriteLine(o);
			}
			catch
			{
				log.WriteLine("Could not write misc data");
			}
			#endregion

			Global.Log.Close();
			this.Shutdown();
		}
		protected override void OnSessionEnding(SessionEndingCancelEventArgs e)
		{
			base.OnSessionEnding(e);

			Global.Log.Close();
		}
	}
}
