﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using CoreLib.Io;
using MapUtility;
using StreamWriter = System.IO.StreamWriter;

namespace ObjectEditor
{
	public class ProvinceTable
	{
		private string header;
		private string path;
		private ProvinceEntry[] entries;
		public ProvinceTable(string path)
		{
			this.path = path;
			List<string> lines = File.ReadAllLines(path, Encoding.Default);
			header = lines[0];
			entries = new ProvinceEntry[lines.Count - 1];

			Dictionary<string, string> provNames = new Dictionary<string, string>(3000);
			string provPath = Settings.GamePath + @"\config\province_names.csv";
			List<string> configLines = File.ReadAllLines(provPath, Encoding.Default);
			foreach (string s in configLines)
			{
				string[] parts = s.Split(';');
				if (parts.Length < 2) 
					continue;
				if (parts[0] == "") 
					continue;
				if (provNames.ContainsKey(parts[0]))
					continue;
				provNames.Add(parts[0], parts[1]);
				
			}

			for (int i = 1; i < lines.Count; i++)
			{
				entries[i - 1] = new ProvinceEntry(lines[i], provNames);
			}
		}

		public ProvinceEntry FindByID(int id)
		{
			var results = from result in entries where result.ID == id select result;
			return results.First();
		}
		public ProvinceEntry this[int id]
		{
			get
			{
				return FindByID(id);
			}
		}

		public void Save()
		{
			using (StreamWriter sw = new StreamWriter(path, false, Encoding.Default))
			{
				sw.WriteLine(header);
				foreach (ProvinceEntry row in entries)
					sw.WriteLine(row.GetSaveString());
			}
		}
	}
	public class ProvinceEntry : ObservableCollection<ViewableInstance>
	{
		private string[] values;
		public int ID { get; private set; }

		public ProvinceEntry(string line, Dictionary<string, string> provNames)
		{
			values = line.Split(';');

			foreach (ViewableObject obj in ViewableObjects.Instance)
			{
				ViewableInstance instance = new ViewableInstance(obj);
				this.Add(instance);
				int x;
				if (int.TryParse(values[obj.xColumn], out x))
					instance.X = x;

				int y;
				if (int.TryParse(values[obj.yColumn], out y))
					instance.Y = y;

				instance.Shown = true;

				ID = int.Parse(values[0]);

				if (!provNames.ContainsKey(values[ViewableObjects.Instance.ProvinceNameColumn]))
					Name = values[ViewableObjects.Instance.ProvinceNameColumn];
				else
					Name = provNames[values[ViewableObjects.Instance.ProvinceNameColumn]];

				if (obj.IndexColumn != null)
				{
					int index;
					if (int.TryParse(values[obj.IndexColumn.Value], out index))
						instance.Index = index;
				}
			}
		}
		public string GetSaveString()
		{
			foreach (ViewableInstance instance in this)
			{
				values[instance.Source.xColumn] = instance.X.ToString();
				values[instance.Source.yColumn] = instance.Y.ToString();

				if (instance.Source.IndexColumn != null)
				{
					values[instance.Source.IndexColumn.Value] = instance.Index.ToString();
				}
			}
			return string.Join(";", values);
		}

		public string Name { get; private set; }
	}
}