﻿using System.Drawing;
using CoreLib;

namespace ObjectEditor
{
	public class ViewableInstance : PropertyChangedNotifier
	{
		public ViewableObject Source { get; private set; }

		public ViewableInstance(ViewableObject source)
		{
			this.Source = source;
		}

		public string ReflectedName
		{
			get
			{
				return Source.Name;
			}
		}
		public Bitmap ReflectedPic
		{
			get
			{
				return Source.Images[Index];
			}
		}

		private bool shown;
		public bool Shown
		{
			get
			{
				return shown;
			}
			set
			{
				if (shown == value)
					return;
				shown = value;
				NotifyPropertyChanged("Shown");
			}
		}

		private int x;
		public int X
		{
			get
			{
				return x;
			}
			set
			{
				if (x == value)
					return;
				x = value;
				NotifyPropertyChanged("X");
			}
		}

		private int y;
		public int Y
		{
			get
			{
				return y;
			}
			set
			{
				if (y == value)
					return;
				y = value;
				NotifyPropertyChanged("Y");
			}
		}

		private int index;
		public int Index
		{
			get
			{
				return index;
			}
			set
			{
				if (index == value)
					return;
				index = value;
				NotifyPropertyChanged("Index");
			}
		}

		public int Width { get { return Source.Sizes[Index].X; } }
		public int Height { get { return Source.Sizes[Index].Y; } }

		public int XOffset { get { return Source.Offsets[Index].X; } }
		public int YOffset { get { return Source.Offsets[Index].Y; } }

		public bool MultipleImages { get { return Source.Images.Length > 1; } }
	}
}