﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using CoreLib;
using CoreLib.Graphics;
using System.Windows.Input;
using MapUtility;
using Panel = System.Windows.Forms.Panel;
using Point = System.Drawing.Point;
using Size = System.Drawing.Size;

namespace ObjectEditor
{
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>
	public partial class Window1 : Window, INotifyPropertyChanged
	{
		#region Options
		private bool useColorscales;
		public bool UseColorscales
		{
			get { return useColorscales; }
			set
			{
				if (useColorscales != value)
				{
					useColorscales = value;
					OnPropertyChanged("useColorscales");
				}
			}
		}
		private bool showTiles;
		public bool ShowTiles
		{
			get { return showTiles; }
			set
			{
				if (showTiles != value)
				{
					showTiles = value;
					OnPropertyChanged("showTiles");
				}
			}
		}
		private List<Colorscales.Color> colorscales;
		public List<Colorscales.Color> Colorscales { get { return colorscales; } set { colorscales = value; } }
		public Colorscales.Color SelectedColor
		{
			get
			{
				return Lightmap.Scales;
			}
			set
			{
				if (Lightmap.Scales != value)
				{
					Lightmap.Scales = value;
					OnPropertyChanged("Colorscales");
					if (small != null && UseColorscales)
					{
						canvas.Refresh();
						secondaryCanvas.Refresh();
					}
				}
			}
		}
		private int zoom;
		public int Zoom
		{
			get
			{
				return zoom;
			}
			set
			{
				if (zoom == value)
					return;
				zoom = value;
				OnPropertyChanged("Zoom");
			}
		}
		#endregion

		#region Notify change
		public event PropertyChangedEventHandler PropertyChanged;
		private void OnPropertyChanged(string info)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(info));
			}
		}
		#endregion

		ProvinceTable provinces = new ProvinceTable(Settings.GamePath + @"\db\province.csv");
		BoundBox boundBox = new BoundBox(Settings.GamePath + @"\map\boundbox.tbl");

		private ProvinceEntry currentProvince;
		public ProvinceEntry CurrentProvince
		{
			get
			{
				return currentProvince;
			}
			set
			{
				if (currentProvince == value) return;
				currentProvince = value;
				OnPropertyChanged("CurrentProvince");
			}
		}

		private Lightmap small;
		private Lightmap large;
		private Panel canvas;
		private Panel secondaryCanvas;

		#region Initialization
		public Window1()
		{
			InitializeComponent();

			this.Title = Shared.ProductName + " v" + AssemblyEx.GetVersion();
			canvas = new Panel();
			secondaryCanvas = new Panel();

			secondaryHost.Child = secondaryCanvas;
			host.Child = canvas;
			canvas.Paint += canvas_Paint;
			canvas.MouseMove += canvas_MouseMove;
			canvas.Click += canvas_Click;

			secondaryCanvas.Paint += secondaryCanvas_Paint;
			secondaryCanvas.MouseDown += secondaryCanvas_MouseDown;
			secondaryCanvas.MouseUp += secondaryCanvas_MouseUp;
			secondaryCanvas.MouseMove += secondaryCanvas_MouseMove;

		}
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			if (!File.Exists(Settings.GamePath + @"\map\lightmap1.tbl"))
			{
				MessageBox.Show("Lightmap1.tbl was not found. Check that GamePath in settings.txt is correct.\r\n" +
					"The program will now exit.");
				Global.Log.WriteLine("Since lightmap1.tbl not found, now exiting. Searched path: "
					+ Settings.GamePath + @"\map\lightmap1.tbl");
				Global.Log.Flush();
				System.Diagnostics.Process.GetCurrentProcess().Kill();
			}
			Global.Log.Write("Loading colorscales...");
			#region Adding colorscales
			colorscales = new List<Colorscales.Color>();
			colorscales.Add(Lightmap.Colorscales.Plain);
			colorscales.Add(Lightmap.Colorscales.Black);
			colorscales.Add(Lightmap.Colorscales.Blue);
			colorscales.Add(Lightmap.Colorscales.Border);
			colorscales.Add(Lightmap.Colorscales.BorderOutline);
			colorscales.Add(Lightmap.Colorscales.Brown);
			colorscales.Add(Lightmap.Colorscales.DarkBlue);
			colorscales.Add(Lightmap.Colorscales.DarkBrown);
			colorscales.Add(Lightmap.Colorscales.DarkGray);
			colorscales.Add(Lightmap.Colorscales.DarkGreen);
			colorscales.Add(Lightmap.Colorscales.DarkOrange);
			colorscales.Add(Lightmap.Colorscales.DarkRed);
			colorscales.Add(Lightmap.Colorscales.DarkYellow);
			colorscales.Add(Lightmap.Colorscales.Gray);
			colorscales.Add(Lightmap.Colorscales.Green);
			colorscales.Add(Lightmap.Colorscales.LightBlue);
			colorscales.Add(Lightmap.Colorscales.LightBrown);
			colorscales.Add(Lightmap.Colorscales.LightGray);
			colorscales.Add(Lightmap.Colorscales.LightGreen);
			colorscales.Add(Lightmap.Colorscales.LightOrange);
			colorscales.Add(Lightmap.Colorscales.LightRed);
			colorscales.Add(Lightmap.Colorscales.LightYellow);
			colorscales.Add(Lightmap.Colorscales.Orange);
			colorscales.Add(Lightmap.Colorscales.Red);
			colorscales.Add(Lightmap.Colorscales.Water);
			colorscales.Add(Lightmap.Colorscales.White);
			colorscales.Add(Lightmap.Colorscales.Yellow);
			SelectedColor = Lightmap.Colorscales.Plain;
			#endregion
			Global.Log.WriteLine("And successful!");

			if (!Settings.HasFourLightmaps)
				sldZoom.Maximum = 3;

			this.DataContext = this;

			this.Title = Shared.ProductName + " v" + AssemblyEx.GetVersion();

			string path = Settings.GamePath + @"\map";
			Zoom = 1;
			large = new Lightmap(LightmapOption.LightMap1, path);
			large.LoadTileData();


			Global.Log.WriteLine("Exiting Window_Loaded");
		}
		#endregion

		#region Dragging movements
		ViewableInstance currentObject;
		Point offsetFromCenter;
		/// <summary>
		/// The current bounds of the secondary canvas, in pixels, found from the boundbox.
		/// </summary>
		Rectangle secondaryBounds;
		/// <summary>
		/// The current bounds of the secondary canvas, in blocks, found from the boundbox.
		/// </summary>
		Rectangle secondaryBlockBounds;
		
		void secondaryCanvas_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (CurrentProvince == null)
				return;


			UpdateLocationData(large, secondaryCanvas, e.Location);

			if (currentObject == null)
				return;

			Rectangle oldInvalidated = GetInvalidationArea(currentObject);

			currentObject.X = e.Location.X - secondaryCanvas.AutoScrollPosition.X - offsetFromCenter.X + secondaryBounds.X;
			currentObject.Y = e.Location.Y - secondaryCanvas.AutoScrollPosition.Y - offsetFromCenter.Y + secondaryBounds.Y;

			Rectangle invalidated = new Rectangle();
			invalidated.X = e.Location.X - offsetFromCenter.X + currentObject.XOffset;
			invalidated.Y = e.Location.Y - offsetFromCenter.Y + currentObject.YOffset;
			invalidated.Width = currentObject.Width;
			invalidated.Height = currentObject.Height;
			secondaryCanvas.Invalidate(oldInvalidated);
			secondaryCanvas.Invalidate(invalidated);
		}
		void secondaryCanvas_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			currentObject = null;			
		}
		void secondaryCanvas_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (CurrentProvince == null)
				return;

			foreach (ViewableInstance obj in CurrentProvince)
			{
				if (!obj.Shown)
					continue;

				Rectangle objBounds = new Rectangle(secondaryCanvas.AutoScrollPosition.X - 
					secondaryBounds.X + obj.XOffset + obj.X,
					secondaryCanvas.AutoScrollPosition.Y - secondaryBounds.Y + obj.YOffset + obj.Y,
					obj.ReflectedPic.Width, obj.ReflectedPic.Height);

				if (objBounds.Contains(e.Location.X, e.Location.Y))
				{
					Color c = obj.ReflectedPic.GetPixel(e.Location.X - objBounds.X, e.Location.Y - objBounds.Y);
					if (c.A == 0)
						continue;
					currentObject = obj;
					offsetFromCenter = new System.Drawing.Point(e.Location.X - objBounds.X - objBounds.Width / 2,
						e.Location.Y - objBounds.Y - objBounds.Height / 2);
				}
			}
		}
		#endregion

		void secondaryCanvas_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			if (large == null || CurrentProvince == null)
				return;

			secondaryCanvas.AutoScrollMinSize = new System.Drawing.Size(secondaryBounds.Width, secondaryBounds.Height);
			e.Graphics.TranslateTransform(secondaryCanvas.AutoScrollPosition.X, secondaryCanvas.AutoScrollPosition.Y);
			Rectangle rect = e.ClipRectangle;
			rect.Offset(-secondaryCanvas.AutoScrollPosition.X, -secondaryCanvas.AutoScrollPosition.Y);

			if (UseColorscales)
			{
				large.DrawTrueColoredGrayscaleLayer(e.Graphics, rect, secondaryBlockBounds.X, secondaryBlockBounds.Y, 
					secondaryBlockBounds.Right, secondaryBlockBounds.Bottom);
			}
			else
			{
				large.Draw<RgbData>(e.Graphics, rect, Lightmap.ColorByProvinceId, secondaryBlockBounds.X, secondaryBlockBounds.Y,
					secondaryBlockBounds.Right, secondaryBlockBounds.Bottom);
			}

			if (ShowTiles)
				large.DrawTilesImage(e.Graphics, rect, secondaryBlockBounds.X, secondaryBlockBounds.Y);

			foreach (ViewableInstance instance in CurrentProvince)
			{
				if (!instance.Shown)
					continue;
				int drawStartX = instance.X - secondaryBounds.X + instance.XOffset;
				int drawStartY = instance.Y - secondaryBounds.Y + instance.YOffset;
				e.Graphics.DrawImageUnscaled(instance.ReflectedPic, drawStartX, drawStartY);
			}

			Rectangle bounds = boundBox[CurrentProvince.ID];
			bounds.Offset(-secondaryBounds.X, -secondaryBounds.Y);
			e.Graphics.DrawRectangle(LinePen, bounds);
		}

		void canvas_Click(object sender, EventArgs e)
		{
			if (lblOwnerId.Text == "")
			{
				CurrentProvince = null;
				secondaryCanvas.Refresh();
				return;
			}
			CurrentProvince = provinces.FindByID(int.Parse(lblOwnerId.Text));

			#region Set secondary bounds
			Rectangle bounds = boundBox[CurrentProvince.ID];
			int xmin = bounds.X / 32;
			int ymin = bounds.Y / 32;
			int xmax = bounds.Right / 32;
			int ymax = bounds.Bottom / 32;
			xmin--; ymin--; xmax++; ymax++;
			if (xmin < 0) xmin = 0;
			if (ymin < 0) ymin = 0;
			if (xmax > large.Width)
				xmax = large.Width;
			if (ymax > large.Height)
				ymax = large.Height;
			secondaryBlockBounds = Rectangle.FromLTRB(xmin, ymin, xmax, ymax);
			secondaryBounds = Rectangle.FromLTRB(xmin * 32, ymin * 32, xmax * 32, ymax * 32);
			#endregion

			secondaryCanvas.Refresh();
		}

		void UpdateLocationData(Lightmap map, Panel canvas, Point location)
		{
			if (map == null)
				return;

			int x = -canvas.AutoScrollPosition.X + location.X;
			int y = -canvas.AutoScrollPosition.Y + location.Y;

			lblLocation.Text = string.Format("({0}, {1})", x.ToString(), y.ToString());
			lblBlkLoc.Text = string.Format("({0}, {1})", (x / 32).ToString(), (y / 32).ToString());
			MapBlock cur = map[x / 32, y / 32];
			cur.EnsureDecompressed();
			Leaf l = cur.FindCoordinates(x, y);
			if (l == null)
				return;
			lblOwnerId.Text = l.OwnerId.OwnerId.ToString();
			lblGrayscale.Text = l.GrayscaleColor.ToString();
			lblIsBorder.Text = l.OwnerId.IsBorder.ToString();
			if (l.OwnerId.IsBorder)
			{
				lblBorderInfo.Text = string.Format("IsTypeOne={0}, Other={1}",
					l.OwnerId.IsTypeOne.ToString(), l.OwnerId.OtherOwner.ToString());
				lblBorderInfo.Visibility = Visibility.Visible;
			}
			else
				lblBorderInfo.Visibility = Visibility.Collapsed;
			cur.DiscardData();
		}
		void canvas_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			UpdateLocationData(small, canvas, e.Location);
		}
		void canvas_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			if (small == null)
				return;

			e.Graphics.TranslateTransform(canvas.AutoScrollPosition.X, canvas.AutoScrollPosition.Y);
			Rectangle rect = e.ClipRectangle;
			rect.X -= canvas.AutoScrollPosition.X;
			rect.Y -= canvas.AutoScrollPosition.Y;

			if (UseColorscales)
			{
				small.DrawTrueColoredGrayscaleLayer(e.Graphics, rect);
			}
			else
			{
				small.Draw<RgbData>(e.Graphics, rect, Lightmap.ColorByProvinceId);
			}
		}
		private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			ComboBox cmb = sender as ComboBox;
			if (cmb.SelectedItem == null) return;
			SelectedColor = cmb.SelectedItem as Colorscales.Color;
		}

		#region Movement with arrow keys
		void host_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			MoveMapByKeys(canvas, e);
		}
		private void secondaryHost_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			MoveMapByKeys(secondaryCanvas, e);
		}
		void MoveMapByKeys(Panel canvas, KeyEventArgs e)
		{
			const int move = 50;

			Point proposed = new Point(-canvas.AutoScrollPosition.X, -canvas.AutoScrollPosition.Y);

			if (e.Key == Key.Down)
			{
				proposed.Y += move;
			}
			else if (e.Key == Key.Up)
			{
				proposed.Y -= move;
			}
			else if (e.Key == Key.Left)
			{
				proposed.X -= move;
			}
			else if (e.Key == Key.Right)
			{
				proposed.X += move;
			}

			canvas.AutoScrollPosition = proposed;

			e.Handled = true;
		}
		#endregion

		#region Drawing
		private static Font font = new Font("Times New Roman", 11);
		private static SolidBrush ColorBrush = new SolidBrush(System.Drawing.Color.Black);
		private static SolidBrush TransparentBrush = new SolidBrush(System.Drawing.Color.Transparent);
		private static System.Drawing.Pen LinePen = new System.Drawing.Pen(System.Drawing.Color.Black);

		private void UpdateTiles(Graphics g, Rectangle bounds)
		{

			if (chkShowTiles.IsChecked.Value)
			{
				large.DrawTilesImage(g, bounds, secondaryBlockBounds.X, secondaryBlockBounds.Y);
			}
		}

		private void chkShowTiles_Checked(object sender, RoutedEventArgs e)
		{
			secondaryCanvas.Refresh();
		}
		#endregion

		private void ShownChanged(object sender, RoutedEventArgs e)
		{
			CheckBox c = (CheckBox)sender;
			ViewableInstance instance = (ViewableInstance)c.DataContext;
			secondaryCanvas.Invalidate(GetInvalidationArea(instance));
		}

		private void Recenter(object sender, RoutedEventArgs e)
		{
			Button b = (Button)sender;
			Rectangle r = boundBox[CurrentProvince.ID];
			ViewableInstance entry = (ViewableInstance)b.DataContext;

			Rectangle oldRec = GetInvalidationArea(entry);

			entry.X = r.Left + r.Width / 2;
			entry.Y = r.Top + r.Height / 2;

			Rectangle newRec = GetInvalidationArea(entry);
			secondaryCanvas.Invalidate(oldRec);
			secondaryCanvas.Invalidate(newRec);
		}
		Rectangle GetInvalidationArea(ViewableInstance instance)
		{
			return new Rectangle(instance.X - secondaryBounds.X + 
				secondaryCanvas.AutoScrollPosition.X + instance.XOffset, 
				instance.Y - secondaryBounds.Y + secondaryCanvas.AutoScrollPosition.Y + instance.YOffset,
				instance.Width, instance.Height);
		}
		private void Save(object sender, RoutedEventArgs e)
		{
			provinces.Save();
		}

		#region Change viewed image
		private void LeftImage(object sender, RoutedEventArgs e)
		{
			ChangeImage(sender, -1);
		}
		private void RightImage(object sender, RoutedEventArgs e)
		{
			ChangeImage(sender, 1);
		}
		private void ChangeImage(object sender, int addValue)
		{
			Button b = (Button)sender;
			ViewableInstance entry = (ViewableInstance)b.DataContext;

			int wanted = entry.Index + addValue;

			if (wanted < 0)
				wanted = entry.Source.Images.Length - 1;
			if (wanted >= entry.Source.Images.Length)
				wanted = 0;

			entry.Index = wanted;

			Rectangle refresh = new Rectangle(entry.X - secondaryBounds.X
				+ secondaryCanvas.AutoScrollPosition.X + entry.XOffset
				, entry.Y - secondaryBounds.Y + secondaryCanvas.AutoScrollPosition.Y + entry.YOffset
				, entry.Width, entry.Height);

			secondaryCanvas.Invalidate(refresh);
		}
		#endregion

		private void Button_Click(object sender, RoutedEventArgs e)
		{
			LightmapOption option = LightmapOption.LightMap1;
			switch (zoom)
			{
				case 1:
					if (Settings.HasFourLightmaps)
						option = LightmapOption.LightMap4;
					else
						option = LightmapOption.LightMap3;
					break;
				case 2:
					if (Settings.HasFourLightmaps)
						option = LightmapOption.LightMap3;
					else
						option = LightmapOption.LightMap2;
					break;
				case 3:
					if (Settings.HasFourLightmaps)
						option = LightmapOption.LightMap2;
					else
						option = LightmapOption.LightMap1;
					break;
				case 4:
					option = LightmapOption.LightMap1;
					break;
			}
			if (option == large.options) //Sharing
			{
				small = large;
			}
			else
			{
				small = new Lightmap(option, Settings.GamePath + @"\map");
			}

			//Move scrolling
			Size oldSize = canvas.AutoScrollMinSize;
			canvas.AutoScrollMinSize = new System.Drawing.Size(small.Width * 32, small.Height * 32);
			if (oldSize.Width != 0)
			{
				double zoomVal = (double)canvas.AutoScrollMinSize.Width / oldSize.Width;
				Point wantedPosition = new Point((int)(-canvas.AutoScrollPosition.X * zoomVal),
					(int)(-canvas.AutoScrollPosition.Y * zoomVal));
				if (wantedPosition.X > canvas.AutoScrollMinSize.Width - canvas.Width)
					wantedPosition.X = canvas.AutoScrollMinSize.Width - canvas.Width;
				if (wantedPosition.Y > canvas.AutoScrollMinSize.Height - canvas.Height)
					wantedPosition.Y = canvas.AutoScrollMinSize.Height - canvas.Height;
				canvas.AutoScrollPosition = wantedPosition;
			}
			canvas.Refresh();
			secondaryCanvas.Refresh();
		}
	}
}