﻿using System;
using System.Collections.Generic;
using System.Drawing;
using CoreLib.Graphics;
using CoreLib.Io;

namespace MapUtility
{
	public class ColorToId
	{
		private Dictionary<RgbData, ushort> provinceColors = new Dictionary<RgbData, ushort>(3000);

		#region Constructors
		public ColorToId(string path)
		{
			FromFile(path);
		}
		public ColorToId() { }
		public ColorToId(bool loadDefault)
		{
			if (loadDefault)
			{
				if (File.Exists(IdToColor.IdMapPath))
					FromFile(IdToColor.IdMapPath);
				else
				{
					//ConsoleEx.ReportNonfatalError("Missing ids.csv. Will use default table.");
					for (ushort id = 0; id < 4000; id++)
					{
						int result = ((((((((((((id & 0x200) >> 1) | ((id & 0x40) << 3)) |
				 ((id & 8) << 7)) | ((id & 1) << 11)) | ((id & 0x400) >> 6)) | ((id & 0x80) >> 2)) |
				 ((id & 0x10) << 2)) | ((id & 2) << 6)) | ((id & 0x800) >> 11)) | ((id & 0x100) >> 7)) |
				 ((id & 0x20) >> 3)) | ((id & 4) << 1);
						result = (((result & 0xf00) << 12) | ((result & 240) << 8)) | ((result & 15) << 4);
						result = (result | -16777216);
						Color c = Color.FromArgb(result);
						AddValue(new RgbData(c.R, c.G, c.B), id);
					}
				}
			}
		}
		private void FromFile(string path)
		{
			List<string> lines = File.ReadAndCleanLines(path);
			foreach (string s in lines)
			{
				string[] parts = s.Split(';');
				if (parts.Length < 4) return;
				ushort provId = ushort.Parse(parts[0]);
				byte R = byte.Parse(parts[1]);
				byte G = byte.Parse(parts[2]);
				byte B = byte.Parse(parts[3]);
				if (!provinceColors.ContainsValue(provId))
				{
					provinceColors.Add(new RgbData(R, G, B), provId);
				}
			}
		}
		#endregion

		public ushort this[RgbData color]
		{
			get
			{
				try
				{
					return provinceColors[color];
				}
				catch (KeyNotFoundException)
				{
					Console.WriteLine("Color not found in id table!");
					Console.WriteLine(color.ToString());
					Console.WriteLine("Press any key to continue, or forcibly close the program now");
					Console.ReadKey();
					provinceColors.Add(color, 0);
					return 0;
				}
			}
		}
		public ushort this[byte R, byte G, byte B]
		{
			get
			{
				return this[new RgbData(R, G, B)];
			}
		}

		public bool TryGetValue(RgbData key, out ushort value)
		{
			return provinceColors.TryGetValue(key, out value);
		}
		public void AddValue(RgbData key, ushort value)
		{
			provinceColors.Add(key, value);
		}
	}
}