﻿using System.Collections.Generic;
using System.IO;
using CoreLib.Graphics;

namespace MapUtility
{
	public class Colorscales
	{
		public Colorscales()
		{
			string[] lines;
			try
			{
				lines = File.ReadAllLines(Global.RootPath + @"\colorscales.csv");
			}
			catch (FileNotFoundException)
			{
				return;
				//We really can't do anything about it.
				//MessageBox.Show("The file: \"map\\colorscales.csv\" was not found. The application will now exit.");
				//throw;
			}
			int startLine;
			for (int i = 0; i < 26; i++)
			{
				startLine = i * 6 + 2;
				Color toChange = new Color(lines[startLine], lines[startLine + 1], lines[startLine + 2], lines[startLine + 3]);
				switch (i)
				{
					case 0: Black = toChange; break;
					case 1: Green = toChange; break;
					case 2: Orange = toChange; break;
					case 3: LightBlue = toChange; break;
					case 4: LightGray = toChange; break;
					case 5: LightBrown = toChange; break;
					case 6: DarkYellow = toChange; break;
					case 7: Yellow = toChange; break;
					case 8: DarkGray = toChange; break;
					case 9: LightYellow = toChange; break;
					case 10: Gray = toChange; break;
					case 11: DarkGreen = toChange; break;
					case 12: Blue = toChange; break;
					case 13: LightRed = toChange; break;
					case 14: White = toChange; break;
					case 15: DarkBrown = toChange; break;
					case 16: Red = toChange; break;
					case 17: LightGreen = toChange; break;
					case 18: LightOrange = toChange; break;
					case 19: Brown = toChange; break;
					case 20: DarkBlue = toChange; break;
					case 21: DarkRed = toChange; break;
					case 22: DarkOrange = toChange; break;
					case 23: Water = toChange; break;
					case 24: Border = toChange; break;
					case 25: BorderOutline = toChange; break;
				}
			}

			Plain = new Color("255;255;255;0", "255;255;255;0", "255;255;255;0", "0;0;0;64");


			#region Naming
			Black.Name = "Black";
			Green.Name = "Green";
			Orange.Name = "Orange";
			LightBlue.Name = "Light blue";
			LightGray.Name = "Light gray";
			LightBrown.Name = "Light brown";
			DarkYellow.Name = "Dark yellow";
			Yellow.Name = "Yellow";
			DarkGray.Name = "Dark gray";
			LightYellow.Name = "Light yellow";
			Gray.Name = "Gray";
			DarkGreen.Name = "Dark green";
			Blue.Name = "Blue";
			LightRed.Name = "Light red";
			White.Name = "White";
			DarkBrown.Name = "Dark brown";
			Red.Name = "Red";
			LightGreen.Name = "Light green";
			LightOrange.Name = "Light orange";
			Brown.Name = "Brown";
			DarkBlue.Name = "Dark blue";
			DarkRed.Name = "Dark red";
			DarkOrange.Name = "Dark orange";
			Water.Name = "Water";
			Border.Name = "Border";
			BorderOutline.Name = "Border outline";
			Plain.Name = "Plain";
			#endregion

		}
		public class Color
		{
			private List<RgbData> scales = new List<RgbData>(65);
			public Color(string line1, string line2, string line3, string line4)
			{
				for (int i = 0; i < 65; i++)
					scales.Add(new RgbData());
				byte[,] colors = new byte[4, 4];
				string str;
				string[] parts;
				for (byte i = 0; i < 4; i++) //A line in colorscales
				{
					switch (i)
					{
						default: str = line1; break; //case 0
						case 1: str = line2; break;
						case 2: str = line3; break;
						case 3: str = line4; break;
					}
					parts = str.Split(';');
					for (byte i2 = 0; i2 < 4; i2++)
					{
						colors[i, i2] = byte.Parse(parts[i2]);
					}
					scales[colors[i, 3]] = new RgbData(colors[i, 0], colors[i, 1], colors[i, 2]);
				}
				for (int i = 0; i < 3; i++) //Loops three times to fill in the colors in between the four indices
				{
					int begin = colors[i, 3]; //The index of when it begins
					int end = colors[i + 1, 3];
					if (end - begin == 0) continue;
					Stack<byte> red = new Stack<byte>(end - begin);
					Stack<byte> green = new Stack<byte>(end - begin);
					Stack<byte> blue = new Stack<byte>(end - begin);
					for (int rgb = 0; rgb < 3; rgb++) //Rotates for each RGB
					{
						int start = colors[i, rgb];
						float diff = (float)(colors[i + 1, rgb] - colors[i, rgb]) / (float)(end - begin);
						float now;
						int rotations = 0;
						for (int val = begin + 1; val < end; val++)
						{
							rotations++;
							now = start + (rotations * diff);
							switch (rgb)
							{
								case 0: red.Push((byte)now); break;
								case 1: green.Push((byte)now); break;
								case 2: blue.Push((byte)now); break;
							}
						}
					}
					while (red.Count != 0)
					{
						scales[--end] = new RgbData(red.Pop(), blue.Pop(), green.Pop());
					}
					if (colors[0, 3] > 0) //If first index is too big
					{
						for (int i2 = 0; i2 < colors[0, 3]; i2++)
						{
							scales[i2] = new RgbData(colors[0, 0], colors[0, 1], colors[0, 2]);
						}
					}
					if (colors[3, 3] < 64) //And if last index is too small
					{
						for (int i2 = 64; i2 > colors[3, 3]; i2--)
						{
							scales[i2] = new RgbData(colors[3, 0], colors[3, 1], colors[3, 2]);
						}
					}
				}
			}
			public RgbData this[int id]
			{
				get
				{
					return scales[id];
				}
			}
			public string Name { get; set; }
			public override string ToString()
			{
				return Name ?? base.ToString();
			}
		}
		public Color Black { get; private set; }
		public Color Green { get; private set; }
		public Color Orange { get; private set; }
		public Color LightBlue { get; private set; }
		public Color LightGray { get; private set; }
		public Color LightBrown { get; private set; }
		public Color DarkYellow { get; private set; }
		public Color Yellow { get; private set; }
		public Color DarkGray { get; private set; }
		public Color LightYellow { get; private set; }
		public Color Gray { get; private set; }
		public Color DarkGreen { get; private set; }
		public Color Blue { get; private set; }
		public Color LightRed { get; private set; }
		public Color White { get; private set; }
		public Color DarkBrown { get; private set; }
		public Color Red { get; private set; }
		public Color LightGreen { get; private set; }
		public Color LightOrange { get; private set; }
		public Color Brown { get; private set; }
		public Color DarkBlue { get; private set; }
		public Color DarkRed { get; private set; }
		public Color DarkOrange { get; private set; }
		public Color Water { get; private set; }
		public Color Border { get; private set; }
		public Color BorderOutline { get; private set; }
		public Color Plain { get; private set; }
	}
}