﻿using System.Collections.Generic;
using System.Drawing;
using CoreLib.Graphics;
using CoreLib.Io;

namespace MapUtility
{
	public class IdToColor
	{
		public const string IdMapPath = "ids.csv";

		private List<RgbData> provinceColors = new List<RgbData>(3000);

		#region Constructors
		public IdToColor(string path)
		{
			FromFile(path);
		}
		public IdToColor() { }
		public IdToColor(bool addDefault)
		{
			if (addDefault)
			{
				if (File.Exists(IdMapPath))
					FromFile(IdMapPath);
				for (ushort id = 0; id < 4000; id++)
				{
					int result = ((((((((((((id & 0x200) >> 1) | ((id & 0x40) << 3)) | ((id & 8) << 7))
						| ((id & 1) << 11)) | ((id & 0x400) >> 6)) | ((id & 0x80) >> 2)) | ((id & 0x10) << 2)) |
						((id & 2) << 6)) | ((id & 0x800) >> 11)) | ((id & 0x100) >> 7)) | ((id & 0x20) >> 3)) | ((id & 4) << 1);
					result = (((result & 0xf00) << 12) | ((result & 240) << 8)) | ((result & 15) << 4);
					result = (result | -16777216);
					this.provinceColors.Add(Color.FromArgb(result));
				}
			}
		}
		private void FromFile(string path)
		{
			for (int i = 0; i < 4000; i++)
				provinceColors.Add(new RgbData());

			List<string> lines = File.ReadAndCleanLines(path);
			foreach (string s in lines)
			{
				string[] parts = s.Split(';');
				if (parts.Length < 4) return;
				ushort provId = ushort.Parse(parts[0]);
				byte R = byte.Parse(parts[1]);
				byte G = byte.Parse(parts[2]);
				byte B = byte.Parse(parts[3]);
				provinceColors[provId] = new RgbData(R, G, B);
			}
		}
		#endregion

		public RgbData this[int id]
		{
			get
			{
				return provinceColors[id];
			}
		}
	}
}