﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace MapUtility
{
	public class Index
	{
		private int[] tiles = new int[336960];
		private BitmapImage rivers;
		private Dictionary<Point, CroppedBitmap> Final = new Dictionary<Point, CroppedBitmap>(700); //By map blocks
		private Size TilesSize; //Counted by tile blocks
		public void DrawRivers(Rect Boundaries, DrawingContext g, int zoom)
		{
			switch (zoom)
			{
				case 3: zoom = 4; break;
				case 4: zoom = 8; break;
			}
			int xmin = (int)Boundaries.Left / (32 / zoom) - 1; //By map block
			int xmax = (int)Boundaries.Right / (32 / zoom) + 1; //By block
			int ymin = (int)Boundaries.Top / (32 / zoom) - 1; //By block
			int ymax = (int)Boundaries.Bottom / (32 / zoom) + 1; //By block
			if (xmin < 0) xmin = 0;
			if (xmax > Settings.Lm1Width) xmax = Settings.Lm1Width;
			if (ymin < 0) ymin = 0;
			if (ymax > Settings.Lm1Height) ymax = Settings.Lm1Height;
			small.Height = small.Width = 32 / zoom;
			for (int x = xmin; x < xmax; x++) //By map block
			{
				for (int y = ymin; y < ymax; y++) //By map block
				{
					int offset = x * Settings.Lm1Height + y;
					if (tiles[offset] != 0)
					{
						Point seek = new Point(x, y); //By map blocks
						if (!Final.ContainsKey(seek))
						{
							ToCrop.X = 32 * (int)(tiles[offset] % TilesSize.Width); //Tile pixels
							ToCrop.Y = (int)(tiles[offset] / TilesSize.Width) * 32; //Tile pixels
							CroppedBitmap cb = new CroppedBitmap(rivers, ToCrop);
							Final.Add(seek, cb);
						}
						if (Final.ContainsKey(seek)) //Map blocks
						{
							g.DrawImage(Final[seek], new Rect(seek.X * 32 / zoom, seek.Y * 32 / zoom, 32 / zoom, 32 / zoom));
						}
					}
				}
			}
		}
		private static Int32Rect ToCrop = new Int32Rect(0, 0, 32, 32);
		private static Rect small = new Rect(0, 0, 32, 32);
		private static Rect large = new Rect(0, 0, 32, 32); //First two change
		public Index()
		{
			rivers = new BitmapImage(new Uri(@"map/tiles.bmp", UriKind.RelativeOrAbsolute));
			TilesSize = new Size(rivers.Width / 32, rivers.Height/ 32);
			int length = tiles.Length;
			using (BinaryReader br = new BinaryReader(new FileStream(@"map\index.tbl", FileMode.Open)))
			{
				for (int i = 0; i < length; i++)
				{
					tiles[i] = br.ReadInt32();
				}
			}
		}
	}
}