using System;
using System.Collections.Generic;
using CoreLib.Io;
using StreamWriter = System.IO.StreamWriter;
using System.Linq;

namespace MapUtility
{
	public class NavalDist
	{
		const int BlockLength = 1500;

		string path;
		NavalDistProvince[] provinces;
		public int NumProvinces { get; private set; }

		public NavalDist(string path)
		{
			this.path = path;

			using (ByteReader br = new ByteReader(path, true))
			{
				NumProvinces = br.ReadInt32();
				provinces = new NavalDistProvince[NumProvinces];

				for (int i = 0; i < NumProvinces; i++)
					provinces[i] = new NavalDistProvince(br, BlockLength, NumProvinces);
			}
		}

		public void ExportData(string dest)
		{
			using (StreamWriter writer = new StreamWriter(dest, false))
			{
				writer.WriteLine("NumProvinces: " + NumProvinces.ToString());
				writer.WriteLine();

				for (int i = 0; i < NumProvinces; i++)
				{
					writer.WriteLine("Province " + i.ToString());
					NavalDistProvince cur = provinces[i];
					//if (cur.All(entry => entry.value == 0))
					//	continue;
					for (int i2 = 0; i2 < NumProvinces; i2++)
					{
						//int value = cur[i2].value;
						//int hi = value >> 16;
						//int low = value & ((255) | (255 << 8));
						//writer.WriteLine("{0}:\tlo:{1}\thi:{2}", i2.ToString(), low.ToString(), hi.ToString());
						//if (cur[i2].value == 0)
						//	writer.WriteLine("{0}: \t{1}", i2.ToString(), cur[i2].value);
						//NavalDistEntry entry = cur[i2];

						NavalDistEntry entry = cur[i2];
						writer.WriteLine("{0}:\tlo: {1}\thi: {2}", i2.ToString(), entry.low.ToString(), entry.high.ToString());
					}
					writer.WriteLine();
				}
			}
		}
	}
	public class NavalDistProvince : IEnumerable<NavalDistEntry>
	{
		NavalDistEntry[] entries;

		public NavalDistProvince(IStreamReader dataStream, int blockLength, int numEntries)
		{
			entries = new NavalDistEntry[numEntries];

			for (int i = 0; i < numEntries; i++)
			{
				int low = dataStream.ReadUInt16();
				int high = dataStream.ReadUInt16();
				entries[i] = new NavalDistEntry(low, high);

				//int val = dataStream.ReadInt32();
				//entries[i] = new NavalDistEntry(val);
			}

			dataStream.Position += (blockLength - numEntries) * 4;
		}

		public NavalDistEntry this[int pos]
		{
			get
			{
				return entries[pos];
			}
		}

		#region IEnumerable<NavalDistEntry> Members

		public IEnumerator<NavalDistEntry> GetEnumerator()
		{
			for (int i = 0; i < entries.Length; i++)
			{
				yield return entries[i];
			}
		}

		#endregion

		#region IEnumerable Members

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return entries.GetEnumerator();
		}

		#endregion
	}
	public struct NavalDistEntry
	{
		public int low;
		public int high;
		//public int value;

		//public NavalDistEntry(int value)
		//{
		//	this.value = value;
		//}

		public NavalDistEntry(int low, int high)
		{
			this.low = low;
			this.high = high;
		}

		public override string ToString()
		{
			//return value.ToString();
			return string.Format("Low: {0} High: {1}", low.ToString(), high.ToString());
		}
	}
}